
# This script explores the effect of missing values on data analyses
# It also explores other strange occurrences in R.
#
# created 11/5/2010 by M. Frazier
#
# last modified: 6/9/2014
#            by: R. Clark
#######################################################################

# we will make a fake data frame.
# rnorm is a function that draws random samples from a
# population with a normal distribution with a given mean
# and standard deviation.  In this case, we are assigning "dogs"
# an average weight of 45 and cats and average weight of 15.

data <- data.frame(animal = rep(c("dog", "cat"), each=5),
                   weight = c(rnorm(5, mean=45, sd=5), rnorm(5, mean=15, sd=2)))

# you can check out our beautiful creation:
data

# now we will replace row 6, column 2 with a missing value
data[6,2] <- NA
data
# That looks more like a real dataframe!
# Let's get the means for each animal type using tapply:

##################################################################
### NA Problem #1: summary data with NA values
##################################################################

tapply(data$weight, data$animal, mean)
# why are the results for cat "NA"??!
# R takes a conservative approach to missing data values.
# Basically it is saying we can't say what the mean is because
# we don't have all the data.

# if you type in ?mean, you will see one of the built in options is na.rm.
# many functions have specialized options for dealing with NA values.

# We can use that here to address this problem:
tapply(data$weight, data$animal, mean, na.rm=TRUE)

##################################################################
### NA Problem #2: subsetting data to remove NA values.
##################################################################

# You can also subset the dataframe so NA values are excluded:

# Let's say that we wanted a subsetted dataframe with only dogs,
# this is how we would do it (remember [rows, columns]...
# and blanks in either of these tells R to use all of them):

dataDogs <- data[data$animal == "dog",]
dataDogs
# or
dataDogs2 <- data[data$animal != "cat",]
dataDogs2

# This might lead you to postulate that the correct way to say you want
# to remove the NA data would be: data[data != NA,].  This does not work:
data[data$weight != NA,]

# THIS IS IMPORTANT...PAY ATTENTION:
# NAs are special and require a special subsetting command (which is annoying).
# Try to remember this so you don't end up having to spend 5 hours someday trying to
# figure this out...

data2 <- data[!is.na(data$weight),]
# Yep that worked...we now have 9 rows of data.

##################################################################
### NA Problem #3: Subsetting data when there are NA values
##################################################################
# Let's replace one of the animals with an NA in the orginal dataset and try subsetting
data[2,1] <- NA
data

# If we want to get all the "cat" data, then:
dataCat <- data[data$animal == "cat",]
# We had 5 cats, so we would expect our new data set to have 5 rows of data
dim(dataCat)
# but it has 6 rows!
# What is going on.
dataCat

# R is including the row with the NA.  Once again, R is taking a conservative approach to
# missing data.  It is saying that we don't know whether or not it is a cat...so it throws it
# in.  This has caused me endless grief.

# My solution:  I never subset data using ==.  Instead, I use %in% because it treats the data
# in a way that is much more intuitive to me.
dataCat <- data[data$animal %in% "cat",]
dim(dataCat)
dataCat

# Another way is to use the "which" function:
# this provides the row numbers where cat is:
which(data$animal == "cat")

# Then you embed this in your subsetting brackets
data[which(data$animal == "cat"),]

# The which statement treats NA values like you would expect.

##################################################################
### NA Problem #4: NA values in regression models
##################################################################

# an ANoVA model is probably a better descriptor than a regression model
# given that the predictors are categorical.  But basically, it is all the same:

mod <- lm(weight ~ animal, data=data)
summary(mod)

# This basically worked, the problem is that if you go to get residuals or predicted values
# that are generated by the model:

predict(mod)
# If you count them, there are only N = 8, or:
length(predict(mod))

# This is a problem if you want to plot the original values vs. the fitted values:
plot(predict(mod), data$weight)

#arghh!  It's not possible because the datasets are no longer matched.

# To get around this:

mod <- lm(weight ~ animal, data=data, na.action=na.exclude)
summary(mod)
predict(mod)
plot(predict(mod), data$weight)

# including na.action=na.exclude pads the output from the resid and predict functions
# so they are now consistent with the orginal data that was used in the models.

##################################################################
### Other useful commands for dealing with NA values
##################################################################

# location of missing data
which(is.na(data$animal))

# sums up NAs in each column
apply(apply(data, 2, is.na), 2, sum)


# remove all rows with NA values
na.omit(data)


##################################################################################
# FINAL NOTES: I have made my biggest errors in R because of NA values.
# I recommend running through this script about every 6 months to make sure
# you are dealing with them correctly!
#################################################################################



#############################################################
## EXTRA CREDIT:  Why is this happening?
#############################################################

data <- na.omit(data)

# If we realized that the last cat should have been classified
# as a dog, we can change that:

# One weird thing is that the rows go to 10, but notice that rows
# 2 and 6 are missing.
data[8,1] <- "dog"

data

# What if instead of being "dog" it should have been "hamster"?
data[8,1] <- "hamster"


# This "invalid factor level" error is very common.  But why are you getting it?

# HINT:  It has to do with the fact that this is a factor.  When you look at this
# variable, you see: "dog", "dog", "dog", "cat", "cat"....
# When R looks at this variable it sees: 1, 1, 1, 2, 2.... and then it refers to
# a look up list that looks like this: 1 = "dog", 2 = "cat".
# When you tried to add "hamster" it didn't know what to do with that because it was
# not on the look up list.

# Question 1: Why are factors considered a very efficient form of data storage?
# Question 2: How can you get "hamster" on that list?

# See file "Answers" for the answers!
